#
# This is the SPEC file for creating binary and source RPMs for the VMs.
#
#
# The Qubes OS Project, http://www.qubes-os.org
#
# Copyright (C) 2010  Joanna Rutkowska <joanna@invisiblethingslab.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#


# default value in case of no qubes-builder's one

Name:		qubes-gui-agent
Version:	4.2.11
Release:	1%{?dist}
Summary:	The Qubes GUI Agent for AppVMs

Group:		Qubes
Vendor:		Invisible Things Lab
License:	GPL
URL:		https://www.qubes-os.org


%define pa_ver_full %((pkg-config --modversion libpulse 2>/dev/null || echo 0.0) | cut -d "-" -f 1)
%define pa_module_dir %(pkg-config --variable=modlibexecdir libpulse)
%if 0%{?is_opensuse}
%define xinitrc_dir /usr/etc/X11/xinit/xinitrc.d
%define limits_dir /usr/etc/security/limits.d
%else
%define xinitrc_dir /etc/X11/xinit/xinitrc.d
%define limits_dir /etc/security/limits.d
%endif

BuildRequires:	gcc
BuildRequires:	libX11-devel
BuildRequires:	libXcomposite-devel
BuildRequires:	libXdamage-devel
BuildRequires:	libXfixes-devel
BuildRequires:	libXt-devel
BuildRequires:	libtool-ltdl-devel
BuildRequires:	libtool
%if 0%{?is_opensuse}
BuildRequires:	libpulse-devel >= 0.9.21
BuildRequires:	libpulse-devel <= 16.99.1
BuildRequires:	xorg-x11-server-sdk
BuildRequires:	libgbm-devel
%else
BuildRequires:	pulseaudio-libs-devel >= 0.9.21, pulseaudio-libs-devel <= 16.99.1
BuildRequires:	xorg-x11-server-devel
BuildRequires:	mesa-libgbm-devel
%endif
BuildRequires:	pam-devel
BuildRequires:	qubes-libvchan-devel
BuildRequires:	qubes-gui-common-devel >= 4.1.0
BuildRequires:	qubes-db-devel
BuildRequires:	xen-devel
BuildRequires:	systemd-rpm-macros
%if 0%{?is_opensuse}
# for directory ownership
BuildRequires:  xinit
BuildRequires:	qubes-core-qrexec
BuildRequires:	qubes-core-agent
Requires(post): %fillup_prereq
%endif
Requires:	Xorg %(xserver-sdk-abi-requires ansic)
Requires:	Xorg %(xserver-sdk-abi-requires videodrv)
Requires:	qubes-core-vm >= 3.0.14
Requires:	xorg-x11-xinit
# qubesdb-read --wait option
Requires:	qubes-db >= 4.1.4
Requires:	python%{python3_pkgversion}-xcffib
Requires:   xorg-x11-server-Xorg
Requires:   xorg-x11-server-Xephyr
Requires:   setxkbmap
Requires:   xsetroot
Requires:   xrdb
Requires:   xmodmap
Requires:   xev
Requires:   xdpyinfo
Requires:   xprop
Requires:   xrandr

Provides:   qubes-gui-vm = %{version}-%{release}
Obsoletes:  qubes-gui-vm < 4.0.0

Source0: %{name}-%{version}.tar.gz

Requires: (%{name}-selinux if selinux-policy)

%package -n pulseaudio-qubes
Summary: Audio support for Qubes VM
# The vchan sink needs .h files from pulseaudio sources
# that are not exported by any *-devel packages; thus they are internal and
# possible to change across version. They are copied to gui git.
# It is possible that our code will work fine with any later pulseaudio
# version; but this needs to be verified for each pulseaudio version.
Requires:	pulseaudio = %{pa_ver_full}
# Needed so that systemd can manage PulseAudio
Conflicts:	qubes-core-agent < 4.2.5
Conflicts:  qubes-gui-vm < 4.0.0
%if 0%{?is_opensuse}
# for directory ownership
BuildRequires: pulseaudio
%endif

%description -n pulseaudio-qubes
Pulseaudio module to enable sound support in Qubes VM

%package -n pipewire-qubes
Summary:       Experimental PipeWire audio support for Qubes VMs
Requires:      pipewire
BuildRequires: pipewire-devel
BuildRequires: qubes-db-devel
BuildRequires: qubes-libvchan-devel
%if 0%{?is_opensuse}
# for directory ownership
BuildRequires: pipewire
%endif

%description -n pipewire-qubes
Experimental PipeWire module to enable sound support in Qubes VMs.
Unlike the PulseAudio support in the pulseaudio-qubes package, this
works with the stock PipeWire service and does not depend on internal
PipeWire headers.  It also promises lower latency than PulseAudio,
at least once the audio daemon supports low latency.

%package xfce
Summary: XFCE desktop support for Qubes VM

%description xfce
XFCE desktop support for Qubes VM

%package -n qubes-gui-vnc
Summary: Remote GuiVM support using VNC
Requires: x11vnc
%if 0%{?is_opensuse}
# for directory ownership
BuildRequires: systemd
%endif

%description -n qubes-gui-vnc
Remote GuiVM support using VNC

%description
The Qubes GUI agent that needs to be installed in VM in order to provide the
Qubes fancy GUI.

%prep
%setup -q

rm -f pulse/pulsecore
ln -s pulsecore-%{pa_ver_full} pulse/pulsecore

%build
%{?set_build_flags}
#make clean
make appvm

%install
rm -rf -- "$RPM_BUILD_ROOT"
make install "DESTDIR=$RPM_BUILD_ROOT" \
                     LIBDIR=%{_libdir} \
                     DATADIR=%{_datadir} \
                     SYSLIBDIR=/usr/lib \
                     XINITRCDIR=%{xinitrc_dir} \
                     LIMITSDIR=%{limits_dir} \
                     PA_VER_FULL=%{pa_ver_full} \
                     ENABLE_SELINUX=%{with selinux} \
                     UNITDIR=%{_unitdir} \
                     USERUNITDIR=%{_userunitdir} \
                     USERPRESETDIR=%{_userpresetdir} \
                     UDEVRULESDIR=%{_udevrulesdir}

%if 0%{?is_opensuse}
mkdir -p $RPM_BUILD_ROOT/usr/lib/pam.d
mv $RPM_BUILD_ROOT/etc/pam.d/* $RPM_BUILD_ROOT/usr/lib/pam.d/
mkdir -p $RPM_BUILD_ROOT%{_fillupdir}
mv $RPM_BUILD_ROOT/etc/sysconfig/desktop \
    $RPM_BUILD_ROOT%{_fillupdir}/desktop.qubes-gui-agent
%endif

%post
if [ -x /bin/systemctl ] && readlink /sbin/init | grep -q systemd; then
    /bin/systemctl enable qubes-gui-agent.service 2> /dev/null
fi

%if 0%{?is_opensuse}
%{fillup_and_insserv}
%endif

%post -n pulseaudio-qubes

sed -i '/^autospawn/d' /etc/pulse/client.conf
echo autospawn=no >> /etc/pulse/client.conf

# simplified version of preset_units() from core-agent-linux
preset_units() {
    local represet=
    cat "$1" | while read action unit_name
    do
        if [ "$action" = "#" -a "$unit_name" = "Units below this line will be re-preset on package upgrade" ]
        then
            represet=1
            continue
        fi
        echo "$action $unit_name" | grep -q '^[[:space:]]*[^#;]' || continue
        [ -n "$action" -a -n "$unit_name" ] || continue
        if [ "$2" = "initial" -o "$represet" = "1" ]
        then
            systemctl --no-reload --global preset "$unit_name" >/dev/null 2>&1 || :
        fi
    done
}

if [ $1 -eq 1 ]
then
    preset_units /usr/lib/systemd/user-preset/75-pulseaudio-qubes.preset initial
else
    preset_units /usr/lib/systemd/user-preset/75-pulseaudio-qubes.preset upgrade
fi

%postun -n pulseaudio-qubes

for unit_name in pipewire.socket pipewire.service \
                 pipewire-pulse.socket pipewire-pulse.service \
                 wireplumber.service; do
    systemctl --no-reload --global preset "$unit_name" >/dev/null 2>&1 || :
done

%post -n pipewire-qubes
%systemd_user_post pipewire.service pipewire.socket wireplumber.service

%preun -n pipewire-qubes
%systemd_user_preun pipewire.service pipewire.socket wireplumber.service

%preun
if [ "$1" = 0 ] ; then
	chkconfig qubes-gui-agent off
    [ -x /bin/systemctl ] && /bin/systemctl disable qubes-gui-agent.service
    /usr/bin/glib-compile-schemas %{_datadir}/glib-2.0/schemas &> /dev/null || :
fi

%posttrans
    /usr/bin/glib-compile-schemas %{_datadir}/glib-2.0/schemas &> /dev/null || :

if [ -d '/etc/init.d' ] && ! [ "$(ls -A /etc/init.d/ 2> /dev/null)" ]; then
    rmdir /etc/init.d
fi

%triggerin -n pulseaudio-qubes -- pulseaudio-libs

sed -i '/^autospawn/d' /etc/pulse/client.conf
echo autospawn=no >> /etc/pulse/client.conf

%clean
rm -rf $RPM_BUILD_ROOT
rm -f %{name}-%{version}

%files
%defattr(-,root,root,-)
%_bindir/qubes-gui
%_bindir/qubes-gui-runuser
%_bindir/qubes-session
%_bindir/qubes-run-xorg
%_bindir/qubes-run-xephyr
%_bindir/qubes-start-xephyr
%_bindir/qubes-change-keyboard-layout
%_bindir/qubes-set-monitor-layout
%_libdir/xorg/modules/drivers/qubes_drv.so
%_libdir/xorg/modules/drivers/dummyqbs_drv.so
%_libdir/libxf86-qubes-common.so
%attr(0644,root,root) /etc/X11/xorg-qubes.conf.template
/etc/profile.d/qubes-gui.sh
/etc/profile.d/qubes-gui.csh
%if 0%{?is_opensuse}
/usr/lib/pam.d/qubes-gui-agent
%else
%config /etc/pam.d/qubes-gui-agent
%endif
%config %{limits_dir}/90-qubes-gui.conf
%config /etc/xdg/Trolltech.conf
%{xinitrc_dir}/20qt-x11-no-mitshm.sh
%{xinitrc_dir}/20qt-gnome-desktop-session-id.sh
%{xinitrc_dir}/50guivm-windows-prefix.sh
/etc/qubes-rpc/qubes.SetMonitorLayout
/etc/qubes-rpc/qubes.GuiVMSession
/etc/qubes/post-install.d/20-qubes-guivm-gui-agent.sh
%if 0%{?is_opensuse}
%{_fillupdir}/desktop.qubes-gui-agent
%else
%config /etc/sysconfig/desktop
%endif
%_unitdir/qubes-gui-agent.service
%_udevrulesdir/70-master-of-seat.rules
%_udevrulesdir/90-qubes-virtual-input-device.rules
%_sysctldir/30-qubes-gui-agent.conf
/usr/lib/qubes/gtk4-workarounds.sh
/usr/lib/qubes/qubes-gui-agent-pre.sh
/usr/lib/qubes/qubes-keymap.sh
/usr/lib/qubes/qubes-xorg-wrapper
%{_datadir}/glib-2.0/schemas/20_qubes-gui-vm.gschema.override
/usr/lib/qubes/icon-sender
/etc/xdg/autostart/qubes-gtk4-workarounds.desktop
/etc/xdg/autostart/qubes-icon-sender.desktop
/etc/xdg/autostart/qubes-qrexec-fork-server.desktop
/etc/xdg/autostart/qubes-keymap.desktop
%dir /var/log/qubes

%files -n pulseaudio-qubes
%_tmpfilesdir/qubes-pulseaudio.conf
/etc/pulse/qubes-default.pa
%_bindir/start-pulseaudio-with-vchan
%{pa_module_dir}/module-vchan-sink.so
/etc/xdg/autostart/qubes-pulseaudio.desktop
/usr/lib/systemd/user-preset/75-pulseaudio-qubes.preset
%dir /usr/lib/systemd/user/pulseaudio.service.d
/usr/lib/systemd/user/pulseaudio.service.d/30_qubes.conf

%files -n pipewire-qubes
# should be owned by pipewire, but it isn't
%dir %_datadir/pipewire/pipewire.conf.d
%_datadir/pipewire/pipewire.conf.d/30_qubes.conf
%_userpresetdir/74-qubes-vm.preset
%_libdir/pipewire-0.3/libpipewire-module-qubes.so
%dir %_defaultlicensedir/pipewire-qubes
%_defaultlicensedir/pipewire-qubes/COPYING
/etc/qubes/post-install.d/20-qubes-pipewire.sh

# Works with ordinary PipeWire service :)

%files xfce
%{xinitrc_dir}/60xfce-desktop.sh

%files -n qubes-gui-vnc
#/lib/systemd/system/qubes-gui-vncserver.service
%attr(0644,root,root) /etc/X11/xorg-qubes-x11vnc.conf.template
%dir %{_unitdir}/lightdm.service.d
%{_unitdir}/lightdm.service.d/qubes-guivm-vnc.conf
%_bindir/qubes-run-x11vnc

%changelog
* Mon Nov 20 2023 Qubes OS Team <qubes-devel@googlegroups.com>
- For complete changelog see: https://github.com/QubesOS/qubes-gui-agent-linux

* Mon Nov 20 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 12638a4
- version 4.2.11

* Sat Nov 18 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - bd627d2
- rpm: use fillup tool to manage /etc/sysconfig/desktop on openSUSE

* Sat Nov 18 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - f07a2b7
- rpm: put PAM config in /usr/lib/pam.d on openSUSE

* Sat Nov 18 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 407ebc0
- rpm: drop explicit dependency on libXfixes

* Sat Nov 18 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - be67a2b
- Move lightdm.service drop-in into /usr/lib

* Thu Nov 16 2023 Rusty Bird <rustybird@net-c.com> - 0aae614
- vmside: avoid XGetWindowAttributes() call if result isn't used

* Thu Nov 16 2023 Rusty Bird <rustybird@net-c.com> - 7d8f143
- vmside: disambiguate two identical error messages

* Thu Nov 16 2023 Rusty Bird <rustybird@net-c.com> - ae2d083
- vmside: add newline to error messages where it was missing

* Sun Nov 05 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 9bc0456
- rpm: adjust directory ownership and add relevant BR to satisfy the check

* Sun Nov 05 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 047545a
- rpm: install files to /usr/etc when relevant on openSUSE

* Sun Nov 05 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 2271cd7
- pipewire: skip duplicated cflags

* Sun Nov 05 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 87573cf
- rpm: adjust dependencies for openSUSE

* Sun Nov 05 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - f8b51b6
- rpm: fix dist macro usage

* Fri Nov 03 2023 Frédéric Pierret (fepitre) <frederic.pierret@qubes-os.org> - ad277ff
- Add dedicated PAM file for Gentoo

* Mon Oct 30 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - a333786
- version 4.2.10

* Sat Oct 28 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - 3476590
- pipewire: adjust buffer parameters

* Wed Oct 25 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 77aa7b0
- Do not override default clock rate or quantum

* Mon Sep 11 2023 Everett Gally <egally@protonmail.com> - 31c5988
- comment explaining why capslock is excluded from key sending

* Mon Sep 11 2023 Everett Gally <egally@protonmail.com> - 6bfefe9
- remove incorrect comment

* Mon Sep 11 2023 Everett Gally <egally@protonmail.com> - 827bb91
- added check to see if modmap is null and moved XFreeModifierMap out of the if statement checking if the current keycode is capslock.

* Sun Sep 10 2023 Everett Gally <egally@protonmail.com> - becf0e7
- remove debug line I added in while testing and forgot to remove in the last commit

* Sun Sep 10 2023 Everett Gally <egally@protonmail.com> - f256db3
- fixed caps lock. It was missing the line to flip its bit in last known states and needs to be excluded from the key send below

* Sun Sep 10 2023 Marek Marczykowski-Górecki <marmarek@invisiblethingslab.com> - b2ff128
- version 4.2.9

* Sat Sep 09 2023 Everett Gally <egally@protonmail.com> - ace84b3
- an extra unused wait_fds[2] line made it in after the conflict was resolved. Removed that line since the variable is unused and stops the build

* Sat Sep 09 2023 Everett Gally <egally@protonmail.com> - bb5297d
- removed some unneeded whitespace

* Sat Sep 09 2023 Everett Gally <egally@protonmail.com> - 1753606
- exclude all BTN_* keys

* Sat Sep 09 2023 Everett Gally <egally@protonmail.com> - b53b05e
- made send_event static

* Sat Sep 09 2023 Everett Gally <egally@protonmail.com> - d30714d
- changed group="user" to group="qubes" in udev rules file. Added lines to install the file in Makefile, appvm-scripts/lib/udev/rules.d/90-qubes-virtual-input-device.rules, debian/qubes-gui-agent.install, and rpm_spec/gui-agent.spec.in

* Fri Sep 08 2023 Everett Gally <egally@protonmail.com> - 98d2dba
- completely separated original handle_keypress function from the one designed for the virtual input device. When input device creation is requested, modifier sync now happens through the uinput_fd instead of the xserver_fd. All modifiers are working as expected when switching between qubes.

* Fri Sep 08 2023 Everett Gally <egally@protonmail.com> - 295464b
- renamed the flag createdInputDevice to created_input_device to follow the same naming convention as other variables

* Fri Sep 08 2023 Everett Gally <egally@protonmail.com> - d8ce3ab
- rename output_fd to uinput_fd so it's clearer what the fd is

* Fri Sep 08 2023 Everett Gally <egally@protonmail.com> - f1637d2
- removed lines setting bits for EV_REL and the REL_BITS. I'm not going to get that in this PR and will work on it in a future one

* Thu Sep 07 2023 Frédéric Pierret (fepitre) <frederic@invisiblethingslab.com> - 2b14649
- Update for Fedora 39

* Sat Sep 02 2023 fepitre-bot <fepitre-bot@qubes-os.org> - bfd9e49
- Add pulseaudio-16.99.1 headers

* Thu Aug 31 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 52facec
- qubes-gui-runuser: obtain default user from qubesdb

* Thu Aug 31 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - fde57fa
- Fix unchecked malloc()

* Thu Aug 31 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 6737a90
- Avoid reserved identifier violation

* Thu Aug 31 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 802e63a
- fix some clang warnings

* Thu Aug 31 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - 42bf492
- Always see if module-vchan-sink.so must be rebuilt

* Thu Aug 31 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - ef1ead4
- Switch from select(2) to poll(2).

* Sun Aug 27 2023 Everett Gally <egally@protonmail.com> - e6d5e59
- Modifier sync will now happen all the time, regardless of whether virtual input device creation was requested or not.

* Sun Aug 27 2023 Everett Gally <egally@protonmail.com> - c2f6757
- virtual input device disabled by default. Can be enabled by enabling the qvm-service 'gui-agent-virtual-input-device'

* Sun Aug 27 2023 Everett Gally <egally@protonmail.com> - b7ad65a
- udev rules file that sets the group owner of the virtual input device to user so xinit can access the device when it's created

* Sat Aug 26 2023 Everett Gally <egally@protonmail.com> - aabe25e
- remove some uneeded whitespace and comments I added previously

* Sat Aug 26 2023 Everett Gally <egally@protonmail.com> - 1b0a80a
- add some log_level checks on some of the fprintf statements I added

* Sat Aug 26 2023 Everett Gally <egally@protonmail.com> - 15b15f8
- removed a comment from a function that doesn't exist anymore

* Sat Aug 26 2023 Everett Gally <egally@protonmail.com> - 047edb5
- added my actual email in the commit so the signature will be verified correctly

* Sat Aug 26 2023 Everett Gally <egally@protonmail.com> - c3fe516
- switched to storing the output_fd and createdInputDevice flag in the ghandles

* Sat Aug 26 2023 Everett Gally <egally@protonmail.com> - cd18ba3
- create input device. if fails, will fall back to xdriver. Keyboard events are working. Mouse events are not currently. Having trouble with getting the cursor into the right position.

* Mon Aug 14 2023 Demi Marie Obenour <demi@invisiblethingslab.com> - d475a29
- Ban old-style declarations and definitions

